from PySide6.QtGui import QAction
from PySide6.QtWidgets import QMessageBox, QFileDialog, QMenuBar, QMenu, QDialog
from PySide6.QtCore import QStandardPaths

from ui.settings import SettingsDialog
from ui.import_preview import PreviewDialog
from ui.editor import BookEditor, MemberEditor

from utils.errors import ExportError, ExportFileError, InvalidContentsError
from services import book_service, book_overview_service


class MenuBar(QMenuBar):
    def __init__(self, parent):
        super().__init__(parent)

        self.parent = parent
        self.file_types = {
            "XML files (*.xml)": ".xml",
            "Any file type (*)": ""
        }

        self.create_file_menu()
        self.create_edit_menu()
        self.create_help_menu()

    def create_file_menu(self):
        # File menu
        file_menu = self.addMenu("File")

        # New submenu
        new_submenu = QMenu("New", self)
        file_menu.addMenu(new_submenu)

        # New book action
        new_book_action = QAction("New book", self)
        new_book_action.triggered.connect(self.new_book)
        new_submenu.addAction(new_book_action)

        # New member action
        new_member_action = QAction("New member", self)
        new_member_action.triggered.connect(self.new_member)
        new_submenu.addAction(new_member_action)

        # Import submenu
        import_submenu = QMenu("Import", self)
        file_menu.addMenu(import_submenu)

        import_books_action = QAction("Import books", self)
        import_books_action.triggered.connect(self.import_books)
        import_submenu.addAction(import_books_action)

        import_members_action = QAction("Import members", self)
        import_members_action.triggered.connect(self.import_members)
        import_submenu.addAction(import_members_action)

        # Export submenu
        export_submenu = QMenu("Export", self)
        file_menu.addMenu(export_submenu)

        export_books_action = QAction("Export books", self)
        export_books_action.triggered.connect(self.export_books)
        export_submenu.addAction(export_books_action)

        export_overview_action = QAction("Export overview", self)
        export_overview_action.triggered.connect(self.export_overviews)
        export_submenu.addAction(export_overview_action)

        export_members_action = QAction("Export members", self)
        export_members_action.triggered.connect(self.export_members)
        export_submenu.addAction(export_members_action)

        file_menu.addSeparator()

        exit_action = QAction("Exit", self)
        exit_action.setShortcut("Ctrl+Q")
        exit_action.triggered.connect(self.parent.close)
        file_menu.addAction(exit_action)

    def create_edit_menu(self):
        # Edit menu
        edit_menu = self.addMenu("Edit")

        # Preferences menu
        preferences_action = QAction("Preferences", self)
        preferences_action.setShortcut("Ctrl+,")
        preferences_action.triggered.connect(self.edit_preferences)
        edit_menu.addAction(preferences_action)

    def create_help_menu(self):
        # Help menu
        help_menu = self.addMenu("Help")

        about_action = QAction("About", self)
        about_action.triggered.connect(self.about)
        help_menu.addAction(about_action)

    def edit_preferences(self):
        SettingsDialog(parent=self).exec()

    def new_book(self):
        BookEditor().exec()
        self.parent.refresh_book_cards()

    def new_member(self):
        MemberEditor().exec()
        self.parent.refresh_member_cards()

    def import_books(self):
        self.import_data("Book", None, book_service)

    def import_members(self):
        # self.import_data("Member", memb)
        pass

    def export_books(self):
        self.export_data("Book", book_service)

    def export_overviews(self):
        self.export_data("Book overview", book_overview_service)

    def export_members(self):
        pass

    def about(self):
        QMessageBox.information(
            self, "About", "Library app demonstrating the phantom read problem")

    def import_data(self, import_name: str, preview_dialog, service):
        try:
            home_dir = QStandardPaths.writableLocation(QStandardPaths.HomeLocation)
            file_path, _ = QFileDialog.getOpenFileName(self, "Choose import file", home_dir, ";;".join(self.file_types.keys()))

            if not file_path:
                return  # User canceled

            parsed_data = service.parse_from_xml(file_path)
            if not parsed_data:
                QMessageBox.warning(self, f"No New {import_name}s", f"No new {import_name}s to import.", QMessageBox.Ok)
                return

            # Show preview dialog
            dialog = PreviewDialog(parsed_data, self)
            if dialog.exec() == QDialog.Accepted:
                # User confirmed, proceed with importing
                book_service.create_books(parsed_data)
                QMessageBox.information(self, "Success", "Books imported successfully!", QMessageBox.Ok)
                self.parent.refresh_book_cards()
            else:
                QMessageBox.information(self, "Canceled", "Import was canceled.", QMessageBox.Ok)
        except InvalidContentsError as e:
            QMessageBox.critical(self,
                                 "Invalid file",
                                 "The file you selected is invalid",
                                 QMessageBox.StandardButton.Ok)
        except ImportError as e:
            QMessageBox.critical(self,
                                 "Error importing books",
                                 f"An error occurred when importing books from the file provided: {e}",
                                 QMessageBox.StandardButton.Ok)

    def export_data(self, export_name: str, service):
        try:
            home_dir = QStandardPaths.writableLocation(QStandardPaths.HomeLocation)
            file_path, selected_filter = QFileDialog.getSaveFileName(self,
                                                                     f"Save {export_name} export",
                                                                     home_dir,
                                                                     ";;".join(self.file_types.keys()))
            if file_path:
                selected_filetype = self.file_types[selected_filter]

                if file_path.endswith(selected_filetype):
                    selected_filetype = ""

                service.export_to_xml(file_path + selected_filetype)

        except ExportFileError as e:
            QMessageBox.critical(self,
                                 "Error saving file",
                                 f"Error occurred when saving the exported data: {e}",
                                 QMessageBox.StandardButton.Ok)
        except ExportError as e:
            QMessageBox.critical(self,
                                 f"Error exporting {export_name}s",
                                 f"An error occurred when exporting {export_name}s: {e}",
                                 QMessageBox.StandardButton.Ok)
